// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI64SSA/utils/util_rx_io_mode.c $
// $Rev: 43474 $
// $Date: 2018-08-31 13:51:49 -0500 (Fri, 31 Aug 2018) $

// 16AI64SSA: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai64ssa_rx_io_mode
*
*	Purpose:
*
*		Provide a visual wrapper for the AI64SSA_IOCTL_RX_IO_MODE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		The setting to apply.
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	ai64ssa_rx_io_mode(int fd, int index, int verbose, s32 set, s32* get)
{
	char	buf[128];
	int		errs;
	int		ret;

	if (verbose)
		gsc_label_index("Rx I/O Mode", index);

	ret		= ai64ssa_ioctl(fd, AI64SSA_IOCTL_RX_IO_MODE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			sprintf(buf, "INVALID: %ld", (long) set);
			break;

		case GSC_IO_MODE_PIO:

			strcpy(buf, "PIO");
			break;

		case GSC_IO_MODE_BMDMA:

			strcpy(buf, "BMDMA");
			break;

		case GSC_IO_MODE_DMDMA:

			strcpy(buf, "DMDMA");
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", buf);

	if (get)
		get[0]	= set;

	return(errs);
}


